.table_timers thead th, #tbl_preliminary thead th {
    text-align: center
}

.table_timers tbody td, #tbl_preliminary tbody td {
    text-align: center;
    word-wrap: break-word;
}

.table_timers tbody td:nth-child(1), .table_timers tbody td:nth-child(2), .table_timers tbody td:nth-child(8) {
    white-space: nowrap;
}

#tbl_preliminary tbody td:nth-child(6), #tbl_preliminary tbody td:nth-child(7) {
    white-space: nowrap;
}


/*
.nav-tabs > li {
    font-weight: bold;
}
*/

.flex-container {
    display: flex;
    margin: 0 auto;
}

.flex-container>div {
    align-self: center;
    padding-left: 3px;
}

.flex-container>div:first-child {
    align-self: center;
    padding-left: 20px;
}


#timerboardImgScreenshot {
    max-height: 450px;
}

.nowrap {
    white-space: nowrap
}
