<label for="timerboardTimer">Timer</label>
<p>{{ object }}</p>

<label for="timerboardImgScreenshot">Image</label>
<p>
    {% if object.details_image_url %}
    <a href="{{ object.details_image_url }}" target="_blank">
        <img id="timerboardImgScreenshot" class="img-responsive" alt="details image" src="{{ object.details_image_url }}"/>
    </a>
    {% else %}
    <span class="text-muted">-</span>
    {% endif %}
</p>

<label for="timerboardNotes">Notes</label>
<textarea id="timerboardNotes" class="form-control" rows="3" readonly>{{ object.details_notes }}</textarea>
